import java.util.HashMap;

public class Parser {
    private Room[] rooms; //An array of all created rooms in the game's environment

    /*
    * Creates a Parser object, which processes player input and prints responses.
    * Takes the array of all created Rooms.
    */
    public Parser(Room[] rooms){
        this.rooms = rooms;
    }

    /*
    * Takes the player's input and the current room of the player and calls commands based on that input.
    * Prints out the result of the command, or error text to inform the player they've made a mistake.
    */
    public void parse(String input){
        HashMap<String, String> directions = createDirections();
        Room currentRoom = findCurrentRoom(rooms);

        //Tests to see if the command is one-word
        if(input.indexOf(" ") == -1){
            //If the command is one-word, cycles through supported single-word commands
            if(input.equals("help")){
                help();
            }
            else if(input.equals("map")){
                map();
            }
            else if(input.equals("directions")){
                printDirections();
            }
            else if(input.equals("look") || input.equals("l")){
                look(currentRoom);
            }
            else if(input.equals("inventory") || input.equals("i")){
                inventory(currentRoom);
            }
            else{
                //Tests against the hashmap of possible directions
                boolean isDirection = false;
                for(String key : directions.keySet()){
                    if(key.equals(input)){
                        exit(directions.get(key), currentRoom);
                        isDirection = true;
                    }
                }
                //Prints out error text if the command can't be identified
                if(!isDirection){
                    System.out.println("Sorry, I don't know how to do that!");
                }
            }
        }
        else{
            //Separates multi-word commands into the action (the first word(s) of the input) and the name of the object to which to apply it (the last word)
            String command = getFirstWords(input);
            String name = getLastWord(input);
            //Cycles through supported multi-word commands
            if(command.equals("examine") || command.equals("x") || command.equals("look at")){
                examine(currentRoom, name);
            }
            else if(command.equals("take") || command.equals("pick up")){
                take(currentRoom, name);
            }
            else if(command.equals("drop") || command.equals("put down")){
                drop(currentRoom, name);
            }
            else if(command.equals("open") || command.equals("close")){
                openClose(currentRoom, name, command);
            }
            else if(command.equals("light") || command.equals("turn on") || command.equals("switch on")){
                turnOn(currentRoom, name);
            }
            else if(command.equals("extinguish") || command.equals("turn off") || command.equals("switch off")){
                turnOff(currentRoom, name);
            }
            else if(command.equals("fill") || command.equals("add to") || command.equals("pay") || command.equals("insert into")){
                add(currentRoom, name);
            }
            else if(command.equals("lock") || command.equals("unlock")){
                lockUnlock(currentRoom, name);
            }
            else if(command.equals("cook")){
                cook(currentRoom, name);
            }
            else if(command.equals("pet")){
                pet(currentRoom, name);
            }
            else if(command.equals("feed")){
                feed(currentRoom, name);
            }
            else if(command.equals("eat")){
                eat(currentRoom, name);
            }
            //Prints out error text if the command can't be identified
            else{
                System.out.println("Sorry, I don't know how to do that!");
            }
        }
    }

    /*
    * Returns the substring of 'input' from the beginning to the last space.
    * Precondition: 'input' contains at least one space
    */
    public String getFirstWords(String input){
        int index = input.lastIndexOf(" ");
        return input.substring(0, index);
    }

    /*
    * Returns the substring of 'input' from the last space to the end.
    * Precondition: 'input' contains at least one space
    */
    public String getLastWord(String input){
        int index = input.lastIndexOf(" " );
        return input.substring(index +1);
    }

    /*
    * Finds and returns a Thing in the array of all Rooms based on the given name.
    */
    public Thing findAnywhere(String name){
        for(Room room : rooms){
            for(Thing thing : room.getContents()){
                for(String currName : thing.getAllNames()){
                    if(currName.equals(name)){
                        return thing;
                    }
                }
            }
        }
        for(Thing thing : findCurrentRoom(rooms).findThing("player").getContents()){
            for(String currName : thing.getAllNames()){
                if(currName.equals(name)){
                    return thing;
                }
            }
        }
        return null;
    }

    /*
    * Finds and returns a Room from the array of all Rooms based on the given name.
    */
    public Room findRoom(String name){
        for(Room room : rooms){
            if(room.getName().equals(name)){
                return room;
            }
        }
        return null;
    }

    /* Finds and returns the Room that currently holds the Player object. */
    public Room findCurrentRoom(Room[] rooms){
        for(Room room: rooms){
            if(room.findIndexOfThing("player") != -1){
                room.setKnown();
                return room;
            }
        }
        return null;
    }

    /*
    * Returns text based on conditionals embedded into the String passed in.
    * Recognized syntaxes: [if thing:state]{text to be printed}, [if thing->room]{text to be printed}, and [if room?state]{text to be printed}.
    * To test for a false state, use an exclamation point in front of the state as though it were a boolean variable.
    * Returns a String of all the initial text that is not behind false conditionals.
    * Precondition: 'text' is not null
    */
    public String processText(String text){
        String result = "";
        //Iterates through all conditionals contained in 'text', if any exist
        while(text.indexOf("[if") != -1){
            //Adds any text before the first conditional to the result
            result += text.substring(0, text.indexOf("[if"));
            //Tests for thing:state syntax
            if(text.substring(text.indexOf("[if") + 4, text.indexOf("]")).indexOf(":") != -1){
                //Identifies the thing and state
                String name = text.substring(text.indexOf("[if") + 4, text.indexOf(":"));
                String state = text.substring(text.indexOf(":") + 1, text.indexOf("]"));
                //Adds the text behind the conditional to the result if the state is true (or false if specified) for the given thing
                if(findAnywhere(name) != null && (state.charAt(0) == '!' && !findAnywhere(name).isState(state.substring(1))) ||(state.charAt(0) != '!' && findAnywhere(name).isState(state))){
                    result += text.substring(text.indexOf("{") + 1, text.indexOf("}"));
                }
            }
            //Tests for thing->room syntax
            else if(text.substring(text.indexOf("[if") + 4, text.indexOf("]")).indexOf("->") != -1){
                //Identifies the thing and room
                String name = text.substring(text.indexOf("[if")+4, text.indexOf("->"));
                String roomName = text.substring(text.indexOf("->") + 2, text.indexOf("]"));
                Room room;
                if(roomName.charAt(0) == '!'){
                    room = findRoom(roomName.substring(1));
                }
                else{
                    room = findRoom(roomName);
                }
                //Adds the text behind the conditional to the result if the thing is in the given room
                if((room.findIndexOfThing(name) != -1 && roomName.charAt(0) != '!') || (room.findIndexOfThing(name) == -1 && roomName.charAt(0) == '!')){
                    result += text.substring(text.indexOf("{") + 1, text.indexOf("}"));
                }
            }
            //Else assumes room?state syntax
            else{
                //Identifies the room and state
                String roomName = text.substring(text.indexOf("[if")+4, text.indexOf("?"));
                String state = text.substring(text.indexOf("?") + 1, text.indexOf("]"));
                Room room = findRoom(roomName);
                //Adds the text behind the conditional to the result if the state is true (or false if specified) for the given room
                if((state.charAt(0) == '!' && !room.isState(state.substring(1))) ||(state.charAt(0) != '!' && room.isState(state))){
                    result += text.substring(text.indexOf("{") + 1, text.indexOf("}"));
                }
            }
            //Removes the current conditional from the text before checking for conditionals again
            text = text.substring(text.indexOf("}") + 1);
        }
        //Adds any remaining text to the result and returns it
        return result + text;
    }

    /* Creates and returns a hashmap of all supported directions. */
    public HashMap<String,String> createDirections(){
        HashMap<String, String> directions = new HashMap<String, String>();
            directions.put("north", "north"); directions.put("n", "north");
            directions.put("south", "south"); directions.put("s", "south");
            directions.put("east", "east"); directions.put("e", "east");
            directions.put("west", "west"); directions.put("w", "west");
            directions.put("northeast", "northeast"); directions.put("ne", "northeast");
            directions.put("northwest", "northwest"); directions.put("nw", "northwest");
            directions.put("southeast", "southeast"); directions.put("se", "southeast");
            directions.put("southwest", "southwest"); directions.put("sw", "southwest");
            directions.put("up", "up"); directions.put("u", "up");
            directions.put("down", "down"); directions.put("d", "down");
        return directions;
    }

    /* Prints the help dialogue (including a list of other commands). */
    public void help(){
        System.out.println("This project uses a very simple parser. It supports traditional one-word commands, as well as several multi-word commands. Multi-word commands consist of a verb (which may include a preposition) and an object to which you wish the verb to be applied. A list of supported commands is provided below:");
        System.out.println("    >help: prints the help dialog");
        System.out.println("    >map: prints a map of the game's environment");
        System.out.println("    >north or >n, >southeast or >se, etc.: moves the player to the room in the specified compass direction. Type \"directions\" to see more.");
        System.out.println("    >look or >l: prints the description of the current room");
        System.out.println("    >examine [thing], >x [thing], or >look at [thing]: prints the description of the item specified");
        System.out.println("    >take [thing] or >pick up [thing]: adds the specified item to your inventory");
        System.out.println("    >drop [thing]: drops the specified item into your current room");
        System.out.println("    >open [thing]: opens the specified thing if it can be opened");
        System.out.println("    >lock [thing] or >unlock [thing]: locks or unlocks the specified thing if you have the key");
        System.out.println("    >close [thing]: closes the specified thing if it can be closed");
        System.out.println("    >light [thing], >turn on [thing], or >switch on [thing]: turns on the specifed item if it can be turned on");
        System.out.println("    >extinguish [thing], >turn off [thing], or >switch off [thing]: turns off the specified item if it can be turned off");
        System.out.println("    >fill [thing] or >pay [thing]: adds a compatible item from your inventory to the specified item");
        System.out.println("    >cook [recipe]: cooks a food if you have the recipe and ingredients");
        System.out.println("    >pet [thing]: allows you to pet an animal");
        System.out.println("    >feed [thing]: allows you to feed an animal if you have a compatible food in your inventory");
    }

    /* Prints a list of all supported directions. */
    public void printDirections(){
        System.out.println("These are the directions that you may be able to use in this game:");
        System.out.println("    >north (n)");
        System.out.println("    >south (s)");
        System.out.println("    >east (e)");
        System.out.println("    >west (w)");
        System.out.println("    >northeast (ne)");
        System.out.println("    >northwest (nw)");
        System.out.println("    >southeast (se)");
        System.out.println("    >southwest (sw)");
        System.out.println("    >up (u)");
        System.out.println("    >down (d)");
    }

    /* Prints an ascii map of the game area*/
    public void map(){
        System.out.println("       ___           N");
        System.out.println(processText("      | A[if player->Blue Bedroom]{*}[if player->!Blue Bedroom]{ }|          ʌ"));
        System.out.println("      |___|      W < o > E");
        System.out.println(" ___   _|_           v");
        System.out.println(processText("| B[if player->Billiard Room]{*}[if player->!Billiard Room]{ }|_| C[if player->Upstairs Hallway]{*}[if player->!Upstairs Hallway]{ }|          S"));
        System.out.println("|___| |___|");
        System.out.println("       _|_\\\\         ___ ");
        System.out.println(processText("      | D[if player->Yellow Bedroom]{*}[if player->!Yellow Bedroom]{ }|\\\\       | E[if player->Kitchen]{*}[if player->!Kitchen]{ }|"));
        System.out.println("      |___| \\\\      |___|");
        System.out.println("         ___ \\\\___   _|_ ");
        System.out.println(processText("        | F[if player->Library]{*}[if player->!Library]{ }|_| G[if player->Hallway]{*}[if player->!Hallway]{ }|_| H[if player->Dining Room]{*}[if player->!Dining Room]{ }|"));
        System.out.println("        |___| |___| |___|");
        if(findRoom("Crawl Space").isState("known")){
            System.out.println("       //      _|_\\\\");
            System.out.println(processText("      //      | I[if player->Entryway]{*}[if player->!Entryway]{ }|\\\\"));
            System.out.println("     //       |___| \\\\");
            System.out.println("  __//          ⇓    \\\\__ ");
            System.out.println(processText(" | K[if player->Crawl Space]{*}[if player->!Crawl Space]{ }|               | J[if player->Cellar]{*}[if player->!Cellar]{ }|"));
            System.out.println(" |___|               |___|");
        }
        else{
            System.out.println("               _|_\\\\");
            System.out.println(processText("              | I[if player->Entryway]{*}[if player->!Entryway]{ }|\\\\"));
            System.out.println("              |___| \\\\");
            System.out.println("                ⇓    \\\\__ ");
            System.out.println(processText("                     | J[if player->Cellar]{*}[if player->!Cellar]{ }|"));
            System.out.println("                     |___|");
        }
        System.out.println("*You are here");
        System.out.println();
        System.out.println(processText("A: [if Blue Bedroom?known]{Blue Bedroom       }[if Blue Bedroom?!known]{???                }G: [if Hallway?known]{Hallway}[if Hallway?!known]{???}"));
        System.out.println(processText("B: [if Billiard Room?known]{Billiard Room      }[if Billiard Room?!known]{???                }H: [if Dining Room?known]{Dining Room}[if Dining Room?!known]{???}"));
        System.out.println(processText("C: [if Upstairs Hallway?known]{Upstairs Hallway   }[if Upstairs Hallway?!known]{???                }I: [if Entryway?known]{Entryway}[if Entryway?!known]{???}"));
        System.out.println(processText("D: [if Yellow Bedroom?known]{Yellow Bedroom     }[if Yellow Bedroom?!known]{???                }J: [if Cellar?known]{Cellar}[if Cellar?!known]{???}"));
        System.out.println(processText("E: [if Kitchen?known]{Kitchen            }[if Kitchen?!known]{???                }[if Crawl Space?known]{K: Crawl Space}"));
        System.out.println(processText("F: [if Library?known]{Library            }[if Library?!known]{???                }"));
    }

    /*
    * Moves the player to a room in the given direction if one exists.
    * Otherwise prints error text.
    */
    public void exit(String direction, Room currentRoom){
        Room destination = currentRoom.getExit(direction);
        if(destination != null){
            destination.addThing(currentRoom.removeThing("player"));
            System.out.println(processText(destination.toString()));
        }
        else{
            System.out.println("You can't go that way!");
        }
    }

    /* Prints the player's inventory. */
    public void inventory(Room room){
        Thing player = room.findThing("player");
        System.out.println(processText(player.getFormattedContents()));
    }

    /* Prints the description of the current room. */
    public void look(Room room){
        System.out.println(processText(room.toString()));
    }

    /*
    * Prints the description of the given object if one exists.
    * Otherwise prints error text.
    */ 
    public void examine(Room room, String name){
        //Looks for the thing in the contents of the current Room
        if(room.findIndexOfThing(name) != -1 && room.isState("lit")){
            System.out.println(processText(room.findThing(name).toString()));
        }
        //Looks for the thing in the player's inventory
        else if(room.findThing("player").findIndexOfThing(name) != -1){
            System.out.println(processText(room.findThing("player").findThing(name).toString()));
        }
        else{
            //Looks for the thing in any containers in the given room
            boolean inBox = false;
            if(room.isState("lit")){
                for(int i = 0; i < room.getContents().size(); i++){
                    Thing current = room.getContents().get(i);
                    if(current.getContents() != null && !current.getDefaultName().equals("player") && current.isState("open")){
                        if(current.findIndexOfThing(name) != -1){
                            inBox = true;
                            System.out.println(processText(current.findThing(name).toString()));
                        }
                    }
                }
            }
            //Prints error text if the thing is not in the room, inventory, or any containers
            if(!inBox){
                System.out.println("Sorry, I can't see that!");
            }
        }
    }

    /*
    * Adds the given object to the player's inventory if it exists and can be taken.
    * Otherwise prints error text.
    */
    public void take(Room room, String name){
        //Looks for the thing in the contents of the given room
        if(room.findIndexOfThing(name) != -1 && room.isState("lit")){
            if(room.findThing(name).isMovable()){
                room.findThing("player").addThing(room.removeThing(name));
                room.findThing("player").findThing(name).changeState("unhidden");
                System.out.println("Taken.");
            }
            //Prints error text if the object cannot be taken
            else{
                System.out.println("You can't take that!");
            }
        }
        //Looks for the thing in the contents of the player's inventory
        else if(room.findThing("player").findIndexOfThing(name) != -1){
            System.out.println("You already have that!");
        }
        else{
            //Looks for the thing in any containers in the given room
            boolean inBox = false;
            if(room.isState("lit")){
                for(int i = 0; i < room.getContents().size(); i++){
                    Thing current = room.getContents().get(i);
                    if(current.getContents() != null && !current.getDefaultName().equals("player") && current.isState("open")){
                        if(current.findIndexOfThing(name) != -1){
                            inBox = true;
                            if(current.findThing(name).isMovable()){
                                room.findThing("player").addThing(current.removeThing(name));
                                room.findThing("player").findThing(name).changeState("unhidden");
                                System.out.println("Taken.");
                            }
                            else{
                                System.out.println("You can't take that!");
                            }
                            break;
                        }
                    }
                }
            }
            //Prints error text if the thing is not in the room, inventory, or any containers
            if(!inBox){
                System.out.println("Sorry, I can't see that!");
            }
        }
    }

    /* Removes the given object from the player's inventory and adds it to the contents of the current room if it exists and can be dropped, otherwise prints error text. */
    public void drop(Room room, String name){
        //Looks for the thing in the player's inventory
        if(room.findThing("player").findIndexOfThing(name) != -1){
            Thing player = room.findThing("player");
            if(!room.isState("lit")){
                System.out.println("It's too dark — you could break it!");
            }
            else if(player.findThing(name).isMovable()){
                room.addThing(player.removeThing(name));
                System.out.println("Dropped.");
            }
            else{
                System.out.println("You can't drop that!");
            }
        }
        //Prints error text
        else{
            System.out.println("You don't have that!");
        }
    }

    /*
    * Opens or closes the given object if it exists.
    * Otherwise prints error text.
    */
    public void openClose(Room room, String name, String command){
        //Looks for the thing in the given room
        if(room.findIndexOfThing(name) != -1 && room.isState("lit")){
            if(room.findThing(name).isState("locked")){
                System.out.println("You need to unlock it first!");
            }
            else{
                System.out.println(processText(room.findThing(name).changeState(command)));
            }
            //If the thing has a direction (i.e. it is a door), adds or removes it to the room depending on whether its open or closed
            if(room.findThing(name).isState("open")){
                if(room.findThing(name).getDirection() != null){
                    room.addExit(room.findThing(name).getDirection(), room.findThing(name).getOtherSide());
                }
            }
            else{
                room.removeExit(room.findThing(name).getDirection());
            }
        }
        //Looks for the thing in the player's inventory
        else if(room.findThing("player").findIndexOfThing(name) != -1){
            if(room.findThing("player").findThing(name).isState("locked")){
                System.out.println("You need to unlock it first!");
            }
            else{
                System.out.println(processText(room.findThing("player").findThing(name).changeState(command)));
            }
        }
        //Prints error text
        else{
            System.out.println("Sorry, I can't see that!");
        }
    }

    /*
    * Unlocks the given object with a compatible key from the player's inventory, if one exists.
    * Otherwise prints error text.
    */
    public void lockUnlock(Room room, String name){
        Thing player = room.findThing("player");
        Boolean resultFound = false; //Keeps track of whether or not a response has already been printed
        //Looks for the thing in the given room
        if(room.findIndexOfThing(name) != -1 && room.isState("lit")){
            if(!room.findThing(name).isState("lockable")){
                System.out.println("You can't do that!");
                resultFound = true;
            }
            else{
                //Iterates through the items in the player's inventory
                for(int i = 0; i < player.getContents().size(); i++){
                    String result = room.findThing(name).changeState(player.getContents().get(i).getDefaultName());
                    //Unlocks the thing if anything other than the default text is returned
                    if(!result.equals("You can't do that!")){
                        System.out.println(processText(result));
                        resultFound = true;
                        break;
                    }
                }
            }
        }
        //Looks for the thing in the player's inventory
        else if(player.findIndexOfThing(name) != -1){
            if(!player.findThing(name).isState("lockable")){
                System.out.println("You can't do that!");
                resultFound = true;
            }
            //Iterates through the items in the player's inventory
            else{
                for(int i = 0; i < player.getContents().size(); i++){
                    String result = player.findThing(name).changeState(player.getContents().get(i).getDefaultName());
                    //Unlocks the thing if anything other than the default text is returned
                    if(!result.equals("You can't do that!")){
                        System.out.println(processText(result));
                        resultFound = true;
                        break;
                    }
                }
            }
        }
        //Prints error text
        else{
            System.out.println("Sorry, I can't see that!");
            resultFound = true;
        }
        if(!resultFound){
            System.out.println("You don't have anything to unlock that with!");
        }
    }

    /*
    * Turns on the given object if it exists.
    * Otherwise prints error text.
    */
    public void turnOn(Room room, String name){
        //Looks for the thing in the given room
        if(room.findIndexOfThing(name) != -1 && room.isState("lit")){
            System.out.println(processText(room.findThing(name).changeState("on")));
        }
        //Looks for the thing in the player's inventory
        else if(room.findThing("player").findIndexOfThing(name) != -1){
            System.out.println(processText(room.findThing("player").findThing(name).changeState("on")));
        }
        //Prints error text
        else{
            System.out.println("Sorry, I can't see that!");
        }
    }

    /*
    * Turns off the given object if it exists.
    * Otherwise prints error text.
    */
    public void turnOff(Room room, String name){
        //Looks for the thing in the given room
        if(room.findIndexOfThing(name) != -1 && room.isState("lit")){
            System.out.println(processText(room.findThing(name).changeState("off")));
        }
        //Looks for the thing in the player's inventory
        else if(room.findThing("player").findIndexOfThing(name) != -1){
            System.out.println(processText(room.findThing("player").findThing(name).changeState("off")));
        }
        //Prints error text
        else{
            System.out.println("Sorry, I can't see that!");
        }
    }

    /*
    * Adds a compatible item from your inventory, if one exists, to the specified fuelable machine, if it exists, and turns it on.
    * Otherwise prints error text.
    */
    public void add(Room room, String name){
        Thing player = room.findThing("player");
        Boolean resultFound = false; //Keeps track of whether or not a response has already been printed
        //Looks for the thing in the given room
        if(room.findIndexOfThing(name) != -1 && room.isState("lit")){
            if(!room.findThing(name).isState("fuelable")){
                System.out.println("You can't do that!");
                resultFound = true;
            }
            else{
                //Iterates through the items in the player's inventory
                for(int i = 0; i < player.getContents().size(); i++){
                    String result = room.findThing(name).changeState(player.getContents().get(i).getDefaultName());
                    //Adds the current item to the specified machine if anything other than default text is returned
                    if(!result.equals("You can't do that!")){
                        System.out.println("You put the " + player.getContents().get(i).getDefaultName() + " into the " + name + ".");
                        System.out.println();
                        System.out.println(processText(result));
                        resultFound = true;
                        player.removeThing(player.getContents().get(i).getDefaultName());
                        break;
                    }
                }
            }
        }
        //Looks for the thing in the player's inventory
        else if(player.findIndexOfThing(name) != -1){
            if(!player.findThing(name).isState("fuelable")){
                System.out.println("You can't do that!");
                resultFound = true;
            }
            else{
                //Iterates through the items in the player's inventory
                for(int i = 0; i < player.getContents().size(); i++){
                    String result = player.findThing(name).changeState(player.getContents().get(i).getDefaultName());
                    //Adds the current item to the specified machine if anything other than default text is returned
                    if(!result.equals("You can't do that!")){
                        System.out.println("You put the " + player.getContents().get(i).getDefaultName() + " into the " + name + ".");
                        System.out.println();
                        System.out.println(processText(result));
                        resultFound = true;
                        player.removeThing(player.getContents().get(i).getDefaultName());
                        break;
                    }
                }
            }
        }
        //Prints error text
        else{
            System.out.println("Sorry, I can't see that!");
            resultFound = true;
        }
        if(!resultFound){
            System.out.println("You don't have anything for that!");
        }
    }

    /*
    * Adds a Thing with the name of the given food to the player's inventory if the player has a compatible recipe and ingredients.
    * Removes the ingredients from the player's inventory.
    * Otherwise prints error text.
    */
    public void cook(Room room, String recipeName){
        if(!room.getName().equals("Kitchen")){
            System.out.println("You're not in a kitchen!");
        }
        else{
            Thing player = room.findThing("player");
            Thing recipe = null;
            for(int i = 0; i < player.getContents().size(); i++){
                if(player.getContents().get(i).getDescription().equals(recipeName)){
                    recipe = player.getContents().get(i);
                }
            }
            if(recipe == null){
                System.out.println("You don't have a recipe on hand for that!");
            }
            else{
                boolean hasAllIngredients = true;
                for(String ingredient : recipe.getIngredients()){
                    boolean hasIngredient = false;
                    for(int i = 0; i < player.getContents().size(); i++){
                        for(int j = 0; j < player.getContents().get(i).getAllNames().length; j++){
                            if(player.getContents().get(i).getAllNames()[j].equals(ingredient)){
                                hasIngredient = true;
                            }
                        }
                    }
                    if(!hasIngredient){
                        hasAllIngredients = false;
                        break;
                    }
                }
                if(hasAllIngredients){
                    for(String ingredient : recipe.getIngredients()){
                        player.removeThing(ingredient);
                    }
                    player.addThing(new Thing(new String[]{recipe.getDescription()}, "edible", true, false));
                    System.out.println("You get to work stirring, mixing, and baking, and end up with a " + recipeName + ".");
                }
                else{
                    System.out.println("You don't have all the ingredients for that!");
                }
            }
        }
    }

    /*
    * Improves the mood of the given animal, if it exists and already has a high enough mood.
    * Otherwise prints error text.
    */
    public void pet(Room room, String name){
        if(room.findIndexOfThing(name) != -1){
            System.out.println(room.findThing(name).changeState("pet"));
        }
        else{
            System.out.println("Sorry, I can't see that!");
        }
    }

    /*
    * If the given animal exists and the the player has a compatible food in their inventory, improves the mood of the animal.
    * Removes the food from the player's inventory.
    * Otherwise prints error text.
    */
    public void feed(Room room, String name){
        if(room.findIndexOfThing(name) != -1 && !room.findThing(name).isState("animal")){
            System.out.println("You can't do that!");
        }
        else if(room.findIndexOfThing(name) != -1){
            Thing player = room.findThing("player");
            Thing animal = room.findThing(name);
            Boolean resultFound = false;
            for(int i = 0; i < player.getContents().size(); i++){
                Thing food = player.getContents().get(i);
                for(int j = 0; j < food.getAllNames().length; j++){
                    String result = animal.changeState(food.getAllNames()[j]);
                    if(!result.equals("You can't do that!")){
                        player.removeThing(food.getDefaultName());
                        System.out.println(result);
                        resultFound = true;
                    }
                }
            }
            if(!resultFound){
                System.out.println("You don't have anything it can eat!");
            }
        }
        else{
            System.out.println("Sorry, I can't see that!");
        }
    }

    /*
    * Prints "You're not hungry" if the given Thing exists.
    * Otherwise prints error text.
    */
    public void eat(Room room, String name){
        if(room.findIndexOfThing(name) != -1 || room.findThing("player").findIndexOfThing(name) != -1){
            System.out.println("You're not hungry!");
        }
        else{
            System.out.println("Sorry, I can't see that!");
        }
    }
}