import java.util.ArrayList;
import java.util.HashMap;

public class Room {
    private String name; //The name of the room.
    private String description; //A paragraph description of the room.
    private HashMap<String, Room> exits;
    private ArrayList<Thing> contents; //A list of Things currently contained by the Room.
    private boolean known; //Stores whether or not the room has been entered by the player.

    /*
    * Provides an environment in which the player may interact with other objects.
    * The description is printed on entering the room and lists everything visible
    * in the room (contained in the 'contents' ArrayList).
    */
    public Room(String name, String description){
        this.name = name;
        this.description = description;
        contents = new ArrayList<Thing>();
        exits = new HashMap<String, Room>();
    }

    /* Returns if the room is a given state. Provides an implementation for "known". Returns false for unrecognized states. */
    public boolean isState(String state){
        if(state.equals("lit")){
            return true;
        }
        if(state.equals("known")){
            return known;
        }
        return false;
    }

    /* Sets the room to known. */
    public void setKnown(){
        known = true;
    }

    /* Adds an exit to the Room. */
    public void addExit(String direction, Room exit){
        exits.put(direction, exit);
    }
    
    /* Removes an exit from the Room. */
    public void removeExit(String direction){
        exits.remove(direction);
    }

    /* Returns the Room's exist in the given direction. */
    public Room getExit(String direction){
        return exits.get(direction);
    }

    /* Returns the Room's name */
    public String getName(){
        return name;
    }

    /* Returns the Room's description */
    public String getDescription(){
        return description;
    }

    /* Returns 'contents' */
    public ArrayList<Thing> getContents(){
        return contents;
    }

    /* Searches for and returns the index of a Thing from 'contents' based on the name passed in. Returns -1 if none exists. */
    public int findIndexOfThing(String name){
        for(int i = 0; i < contents.size(); i++){
            for(int j = 0; j < contents.get(i).getAllNames().length; j++){
                if(contents.get(i).getAllNames()[j].toLowerCase().equals(name)){
                    return i;
                }
            }
        }
        return -1;
    }

    /* Searches for and returns a Thing from 'contents' based on the name passed in. Returns null if none exists. */
    public Thing findThing(String name){
        int index = findIndexOfThing(name);
        if(index != -1){
            return contents.get(index);
        }
        return null;
    }

    /* Adds a Thing to 'contents'. */
    public void addThing(Thing thing){
        contents.add(thing);
    }

    /* Removes and returns a Thing from 'contents'. */
    public Thing removeThing(String name){
        return contents.remove(findIndexOfThing(name));
    }

    /* Returns "There's [list of nonhidden contents] here." If there are none, returns null. */
    public String getFormattedContents(){
        int numThings = 0;
        String result = "There's";
        //Tests if 'contents' has objects in it
        if(contents.size() != 0){
            for(int i = 0; i < contents.size(); i++){
                //Tests whether each object is hidden before including it in the String
                if(!contents.get(i).isHidden()){
                    //Tests whether or not the current Thing is the last nonhidden Thing in 'contents'
                    boolean last = true;
                    for(int j = i + 1; j < contents.size(); j++){
                        if(!contents.get(j).isHidden()){
                            last = false;
                            break;
                        }
                    }
                    //Adds a comma before the name if there has already been at least one nonhidden Thing and there is more than two nonhidden Things total
                    if(numThings >=1 && !(numThings == 1 && last)){
                        result += ",";
                    }
                    //Adds an "and" before the last nonhidden Thing in 'contents'
                    if(numThings >= 1 && last){
                        result += " and";
                    }
                    numThings += 1;
                    String name = contents.get(i).getDefaultName();
                    //Adds the appropriate article and then the name of the Thing to the resulting String
                    result += " a";
                    if(name.charAt(0) == 'a' || name.charAt(0) == 'e' ||name.charAt(0) == 'i' || name.charAt(0) == 'o' || name.charAt(0) == 'u'){
                        result += "n";
                    }
                    result += " " + name;
                }
            }
        }
        //If 'contents' had no nonhidden objects, return null.
        if(numThings == 0){
            return null;
        }
        return result += " here.";
    }

    /* Prints the formatted name, description, and list of contents. */
    @Override
    public String toString(){
        if(!name.equals("The End")){
            String result = "**" + getName().toUpperCase() + "**" + "\n" + getDescription();
            if(getFormattedContents() != null){
                result += "\n\n" + getFormattedContents();
            }
            return result;
        }
        else{
            return "**" + getName().toUpperCase() + "**" + "\n" + getDescription();
        }
    }
}