import java.util.ArrayList;

public class Thing {
    private String[] names; //The names of the thing. Names must be unique.
    private String description; //The description of the thing. Should start with a lowercase letter and end without punctuation,
    private boolean movable; //Stores whether or not the thing can be taken and dropped by the player.
    private boolean hidden; //Stores whether or not the thing is to be printed in room descriptions.
    private boolean known; //Stores whether or not the player has ever held the object.

    /*
    * Implements interactive objects that represent physical items in the player's world.
    * A Thing may have multiple names that can be recognized by both the player and the parser,
    * stored in 'names'.
    * Descriptions should fit in the format "The [name] is [description]."
    * The movable flag allows Things to be taken and dropped by the player. The hidden flag
    * hides the Thing from room descriptions.
    */
    public Thing(String[] names, String description, boolean movable, boolean hidden){
        this.names = names;
        this.description = description;
        this.movable = movable;
        this.hidden = hidden;
    }

    /* Returns the default name (stored at index 0 of 'names') to be used in generic text, e.g. room descriptions. */
    public String getDefaultName(){
        return names[0];
    }

    /* Returns the array of names. */
    public String[] getAllNames(){
        return names;
    }

    /* Returns whether the Thing is to be printed in room descriptions. */
    public boolean isHidden(){
        return hidden;
    }

    /* Returns the description. */
    public String getDescription(){
        return description;
    }

    /* Sets 'known' to true. */
    public void setKnown(){
        known = true;
    }

    public boolean isState(String state){
        if(state.equals("known")){
            return known;
        }
        return false;
    }

    /*
    * Changes the hidden state.
    * Returns a String describing the action that happened, or the default message of "You can't do that!"
    */
    public String changeState(String result){
        if(result.equals("hidden") && !hidden){
            hidden = true;
            return "Hidden.";
        }
        else if(result.equals("unhidden") && hidden){
            hidden = false;
            return "Unhidden.";
        }
        return "You can't do that!";
    }

    /* Returns whether the thing can be taken and dropped by the player. */
    public boolean isMovable(){
        return movable;
    }

    /* Returns the Things' description in the format "the [default name] is/are [description]." */
    @Override
    public String toString(){
        String isAre = "is";
        if(getDefaultName().charAt(getDefaultName().length() - 1) == 's'){
            isAre = "are";
        }
        return "The " + getDefaultName() + " " + isAre + " " + description + ".";
    }

    /*
    * Provides a default response for getting the contents of a Thing.
    * Returns null.
    */
    public ArrayList<Thing> getContents(){
        return null;
    }

    /*
    * Provides a default response for getting the ingredients of a Thing.
    * Returns null.
    */
    public String[] getIngredients(){
        return null;
    }

    /*
    * Provides a default response for getting the index of a Thing.
    * Returns -1.
    */
    public int findIndexOfThing(String name){
        return -1;
    }

    /*
    * Provides a default response for getting a Thing.
    * Returns null.
    */
    public Thing findThing(String name){
        return null;
    }

    /*
    * Provides a default response for adding a Thing.
    * Does nothing.
    */
    public void addThing(Thing thing){}

    /*
    * Provides a default response for removing and returning a Thing.
    * Returns null.
    */
    public Thing removeThing(String name){
        return null;
    }

    /*
    * Provides a default response for getting the formatted contents of a Thing.
    * Returns null.
    */
    public String getFormattedContents(){
        return null;
    }

    /*
    * Provides a default response for getting the direction of a Thing.
    * Returns null.
    */
    public String getDirection(){
        return null;
    }

    /*
    * Provides a default response for getting the contents of a Thing.
    * Returns null.
    */
    public Room getOtherSide(){
        return null;
    }
}